@@ -7,6 +7,7 @@ from bert4keras.backend import keras, K, is_tf_keras
 from bert4keras.backend import align, sequence_masking
 from bert4keras.backend import recompute_grad
 from bert4keras.backend import attention_normalize
+from bert4keras.backend import sinusoidal_embeddings
 from bert4keras.backend import apply_rotary_position_embeddings
 from keras import initializers, activations
 from keras.layers import *
@@ -484,16 +485,15 @@ class MultiHeadAttention(Layer):
         kw = self.k_dense(k)
         vw = self.v_dense(v)
         # 形状变换
-        qw = K.reshape(qw, (-1, K.shape(q)[1], self.heads, self.key_size))
-        kw = K.reshape(kw, (-1, K.shape(k)[1], self.heads, self.key_size))
-        vw = K.reshape(vw, (-1, K.shape(v)[1], self.heads, self.head_size))
+        qw = K.reshape(qw, (self.heads, self.key_size), -1)
+        kw = K.reshape(kw, (self.heads, self.key_size), -1)
+        vw = K.reshape(vw, (self.heads, self.head_size), -1)
         # Attention
         qkv_inputs = [qw, kw, vw] + inputs[3:]
         qv_masks = [q_mask, v_mask]
         o, a = self.pay_attention_to(qkv_inputs, qv_masks, **kwargs)
         # 完成输出
-        o = K.reshape(o, (-1, K.shape(o)[1], self.head_size * self.heads))
-        o = self.o_dense(o)
+        o = self.o_dense(K.flatten(o, 2))
         # 返回结果
         if self.return_attention_scores:
             return [o, a]
@@ -531,6 +531,8 @@ class MultiHeadAttention(Layer):
         if self.attention_scale:
             a = a / self.key_size**0.5
         if a_bias is not None:
+            if K.ndim(a_bias) == 3:
+                a_bias = align(a_bias, [0, -2, -1], K.ndim(a))
             a = a + a_bias
         a = sequence_masking(a, v_mask, '-inf', -1)
         A = attention_normalize(a, -1, self.normalization)
@@ -592,6 +594,7 @@ class GatedAttentionUnit(Layer):
         activation='swish',
         use_bias=True,
         normalization='squared_relu',
+        self_attention=True,
         attention_scale=True,
         attention_dropout=None,
         kernel_initializer='glorot_uniform',
@@ -600,9 +603,10 @@ class GatedAttentionUnit(Layer):
         super(GatedAttentionUnit, self).__init__(**kwargs)
         self.units = units
         self.key_size = key_size
-        self.activation = activation
+        self.activation = activations.get(activation)
         self.use_bias = use_bias
         self.normalization = normalization
+        self.self_attention = self_attention
         self.attention_scale = attention_scale
         self.attention_dropout = attention_dropout
         self.kernel_initializer = initializers.get(kernel_initializer)
@@ -613,33 +617,61 @@ class GatedAttentionUnit(Layer):
         hidden_size = input_shape[-1]
         if isinstance(hidden_size, (list, tuple)):
             hidden_size = input_shape[0][-1]
-        self.i_dense = Dense(
-            units=2 * self.units + self.key_size,
-            activation=self.activation,
-            use_bias=self.use_bias,
-            kernel_initializer=self.kernel_initializer
-        )
+        if self.self_attention:
+            self.i_dense = Dense(
+                units=2 * self.units + self.key_size,
+                activation=self.activation,
+                use_bias=self.use_bias,
+                kernel_initializer=self.kernel_initializer
+            )
+            self.q_scaleoffset = ScaleOffset(offset=self.use_bias)
+            self.k_scaleoffset = ScaleOffset(offset=self.use_bias)
+        else:
+            self.uq_dense = Dense(
+                units=self.units + self.key_size,
+                activation=self.activation,
+                use_bias=self.use_bias,
+                kernel_initializer=self.kernel_initializer
+            )
+            self.k_dense = Dense(
+                units=self.key_size,
+                activation=self.activation,
+                use_bias=self.use_bias,
+                kernel_initializer=self.kernel_initializer
+            )
+            self.v_dense = Dense(
+                units=self.units,
+                activation=self.activation,
+                use_bias=self.use_bias,
+                kernel_initializer=self.kernel_initializer
+            )
         self.o_dense = Dense(
             units=hidden_size,
             use_bias=self.use_bias,
             kernel_initializer=self.kernel_initializer
         )
-        self.q_scaleoffset = ScaleOffset(offset=self.use_bias)
-        self.k_scaleoffset = ScaleOffset(offset=self.use_bias)
 
     @recompute_grad
     def call(self, inputs, mask=None, a_bias=None, p_bias=None):
         if not isinstance(inputs, list):
             inputs, mask = [inputs], [mask]
-        x, n = inputs[0], 1
+        if self.self_attention:
+            x, n = inputs[0], 1
+        else:
+            (q, k, v), n = inputs[:3], 3
         mask = None if mask is None else mask[0]
         if a_bias:
             a_bias = inputs[n]
             n += 1
         # 投影变换
-        x = self.i_dense(x)
-        u, v, qk = tf.split(x, [self.units, self.units, self.key_size], axis=-1)
-        q, k = self.q_scaleoffset(qk), self.k_scaleoffset(qk)
+        if self.self_attention:
+            x = self.i_dense(x)
+            u, v, qk = tf.split(x, [self.units, self.units, self.key_size], -1)
+            q, k = self.q_scaleoffset(qk), self.k_scaleoffset(qk)
+        else:
+            uq = self.uq_dense(q)
+            u, q = tf.split(uq, [self.units, self.key_size], -1)
+            k, v = self.k_dense(k), self.v_dense(v)
         # 加入RoPE
         if p_bias == 'rotary':
             q, k = apply_rotary_position_embeddings(inputs[n], q, k)
@@ -676,6 +708,7 @@ class GatedAttentionUnit(Layer):
             'activation': activations.serialize(self.activation),
             'use_bias': self.use_bias,
             'normalization': self.normalization,
+            'self_attention': self.self_attention,
             'attention_scale': self.attention_scale,
             'attention_dropout': self.attention_dropout,
             'kernel_initializer':
@@ -694,7 +727,7 @@ class LayerNormalization(ScaleOffset):
         super(LayerNormalization, self).__init__(**kwargs)
         self.zero_mean = zero_mean
         self.unit_variance = unit_variance
-        self.epsilon = epsilon or 1e-12
+        self.epsilon = epsilon or K.epsilon()
 
     @recompute_grad
     def call(self, inputs):
@@ -833,7 +866,6 @@ class SinusoidalPositionEmbedding(Layer):
         """
         if self.custom_position_ids:
             inputs, position_ids = inputs
-            seq_len = K.shape(inputs)[1]
             if 'float' not in K.dtype(position_ids):
                 position_ids = K.cast(position_ids, K.floatx())
         else:
@@ -841,11 +873,7 @@ class SinusoidalPositionEmbedding(Layer):
             batch_size, seq_len = input_shape[0], input_shape[1]
             position_ids = K.arange(0, seq_len, dtype=K.floatx())[None]
 
-        indices = K.arange(0, self.output_dim // 2, dtype=K.floatx())
-        indices = K.pow(10000.0, -2 * indices / self.output_dim)
-        embeddings = tf.einsum('bn,d->bnd', position_ids, indices)
-        embeddings = K.stack([K.sin(embeddings), K.cos(embeddings)], axis=-1)
-        embeddings = K.reshape(embeddings, (-1, seq_len, self.output_dim))
+        embeddings = sinusoidal_embeddings(position_ids, self.output_dim)
 
         if self.merge_mode == 'add':
             return inputs + embeddings
