@@ -369,7 +369,7 @@ class LM_Mask(object):
                 idxs = K.arange(0, seq_len)
                 mask = idxs[None, :] <= idxs[:, None]
                 mask = K.cast(mask, K.floatx())
-                return -(1 - mask[None, None]) * K.infinity()
+                return -(1 - mask) * K.infinity()
 
             self.attention_bias = self.apply(
                 inputs=self.inputs[0],
@@ -395,7 +395,7 @@ class UniLM_Mask(object):
                 idxs = K.cumsum(s, axis=1)
                 mask = idxs[:, None, :] <= idxs[:, :, None]
                 mask = K.cast(mask, K.floatx())
-                return -(1 - mask[:, None]) * K.infinity()
+                return -(1 - mask) * K.infinity()
 
             self.attention_bias = self.apply(
                 inputs=self.inputs[1],
@@ -513,6 +513,7 @@ class BERT(Transformer):
         x = self.apply(
             inputs=self.simplify([x, z]),
             layer=LayerNormalization,
+            epsilon=1e-12,
             conditional=(z is not None),
             hidden_units=self.layer_norm_conds[1],
             hidden_activation=self.layer_norm_conds[2],
@@ -577,6 +578,7 @@ class BERT(Transformer):
         x = self.apply(
             inputs=self.simplify([x, z]),
             layer=LayerNormalization,
+            epsilon=1e-12,
             conditional=(z is not None),
             hidden_units=self.layer_norm_conds[1],
             hidden_activation=self.layer_norm_conds[2],
@@ -606,6 +608,7 @@ class BERT(Transformer):
         x = self.apply(
             inputs=self.simplify([x, z]),
             layer=LayerNormalization,
+            epsilon=1e-12,
             conditional=(z is not None),
             hidden_units=self.layer_norm_conds[1],
             hidden_activation=self.layer_norm_conds[2],
@@ -666,6 +669,7 @@ class BERT(Transformer):
             x = self.apply(
                 inputs=self.simplify([x, z]),
                 layer=LayerNormalization,
+                epsilon=1e-12,
                 conditional=(z is not None),
                 hidden_units=self.layer_norm_conds[1],
                 hidden_activation=self.layer_norm_conds[2],
@@ -974,7 +978,7 @@ class NEZHA(BERT):
             s = self.apply(
                 inputs=s,
                 layer=Embedding,
-                input_dim=2,
+                input_dim=self.segment_vocab_size,
                 output_dim=self.embedding_size,
                 embeddings_initializer=self.initializer,
                 name=name
@@ -2269,9 +2273,10 @@ class T5_Encoder(T5_Base):
 class T5_Decoder(LM_Mask, T5_Base):
     """Google的T5模型（Decoder）
     """
-    def __init__(self, with_lm=True, **kwargs):
+    def __init__(self, with_lm=True, cross_position_bias=True, **kwargs):
         super(T5_Decoder, self).__init__(**kwargs)
         self.with_lm = with_lm
+        self.cross_position_bias = cross_position_bias
 
     def get_inputs(self):
         """T5的Decoder的输入为context序列和token_ids
@@ -2388,13 +2393,16 @@ class T5_Decoder(LM_Mask, T5_Base):
             hidden_initializer=self.initializer,
             name='%s-Norm' % cross_attention_name
         )
+        if self.cross_position_bias:
+            inputs = [x, c, c, position_bias[1]]
+            arguments = {'a_bias': None, 'p_bias': 't5_relative'}
+        else:
+            inputs = [x, c, c]
+            arguments = {'a_bias': None, 'p_bias': None}
         x = self.apply(
-            inputs=[x, c, c, position_bias[1]],
+            inputs=inputs,
             layer=MultiHeadAttention,
-            arguments={
-                'a_bias': None,
-                'p_bias': 't5_relative'
-            },
+            arguments=arguments,
             heads=self.num_attention_heads,
             head_size=self.attention_head_size,
             out_dim=self.hidden_size,
